@@ -59,7 +59,9 @@
 #include "isochronous.hpp"
 #include "iperf_multicast_api.h"
 #include "Mutex.h"
-
+#if HAVE_UDP_L4S
+#include "prague_cc.h"
+#endif
 /* ------------------------------------------------------------------- */
 class Client {
 public:
@@ -91,6 +93,11 @@ private:
     inline void WritePacketID(intmax_t);
     inline void WriteTcpTxHdr(struct ReportStruct *, int, int);
     inline void WriteTcpTxBBHdr(struct ReportStruct *, uint32_t, int);
+    inline int myWrite(int inSock, const void *inBuf, int inLen);
+    inline int myWriten(int inSock, const void *inBuf, int inLen, int *count);
+#if HAVE_TCP_STATS
+    inline void mygetTcpInfo(void);
+#endif
     inline double get_delay_target(void);
     void InitTrafficLoop(void);
     void SetReportStartTime(void);
@@ -124,6 +131,11 @@ private:
     void RunUDPIsochronous(void);
     // UDP plain
     void RunUDP(void);
+    void RunUDPBurst(void);
+#if HAVE_UDP_L4S
+    void RunUDPL4S(void);
+    int ack_poll (time_tp ack_timeout);
+#endif
     // client connect
     void PeerXchange(void);
     thread_Settings *mSettings;
@@ -146,14 +158,7 @@ private:
 #if HAVE_DECL_SO_MAX_PACING_RATE
     Timestamp PacingStepTime;
 #endif
-#if HAVE_DECL_TCP_TX_DELAY
-    enum delay_state {NO_DELAY=0, ADD_DELAY};
-    delay_state current_state;
-    int state_tokens[2];
-    inline void apply_txdelay_func (void);
-    Timestamp TcpTxDelayQuantumEnd;
-#endif
-
+    struct udp_l4s_ack UDPAckBuf;
 }; // end class Client
 
 #endif // CLIENT_H
