@@ -95,6 +95,8 @@ extern const char client_report_epoch_start[];
 
 extern const char client_report_epoch_start_current[];
 
+extern const char server_report_epoch_start_current[];
+
 extern const char server_pid_port[];
 
 extern const char server_working_load_port[];
@@ -208,6 +210,8 @@ extern const char report_bw_read_enhanced_format[];
 
 extern const char report_sum_bw_read_enhanced_format[];
 
+extern const char report_sumcnt_bw_jitter_loss_enhanced_header[];
+
 extern const char report_sumcnt_bw_read_enhanced_header[];
 
 extern const char report_sumcnt_bw_read_enhanced_format[];
@@ -286,6 +290,8 @@ extern const char report_bw_jitter_loss_enhanced_isoch_triptime_format[];
 
 extern const char report_bw_jitter_loss_suppress_enhanced_format[];
 
+extern const char report_bw_jitter_loss_suppress_isoch_format[];
+
 extern const char report_sum_bw_jitter_loss_enhanced_format[];
 
 extern const char report_sumcnt_bw_jitter_loss_enhanced_format[];
@@ -380,16 +386,29 @@ extern const char reportCSV_peer[];
 
 extern const char reportCSV_bw_format[];
 
-extern const char reportCSV_bw_read_enhanced_header[];
-extern const char reportCSV_bw_read_enhanced_format[];
+extern const char reportCSV_bw_tcp_read_enhanced_header[];
+extern const char reportCSV_bw_tcp_read_enhanced_format[];
 
-extern const char reportCSV_bw_write_enhanced_header[];
-extern const char reportCSV_bw_write_enhanced_format[];
+extern const char reportCSV_bw_tcp_read_triptime_header[];
+extern const char reportCSV_bw_tcp_read_triptime_format[];
+extern const char reportCSV_bw_tcp_read_isochronous_header[];
+
+extern const char reportCSV_bw_tcp_write_enhanced_header[];
+extern const char reportCSV_bw_tcp_write_enhanced_format[];
 
 extern const char reportCSV_bw_jitter_loss_format[];
 
-extern const char reportCSV_bw_jitter_loss_pps_header[];
-extern const char reportCSV_bw_jitter_loss_pps_format[];
+extern const char reportCSV_bw_udp_read_enhanced_header[];
+extern const char reportCSV_bw_udp_read_enhanced_format[];
+
+extern const char reportCSV_bw_udp_read_triptime_header[];
+extern const char reportCSV_bw_udp_read_triptime_format[];
+
+extern const char reportCSV_bw_udp_read_isochronous_header[];
+extern const char reportCSV_bw_udp_read_isochronous_format[];
+
+extern const char reportCSV_bw_udp_write_enhanced_header[];
+extern const char reportCSV_bw_udp_write_enhanced_format[];
 
 extern const char reportCSV_client_bb_bw_tcp_header[];
 extern const char reportCSV_client_bb_bw_tcp_format[];
