@@ -1,5 +1,5 @@
 /*---------------------------------------------------------------
- * Copyrighta (c) 2017
+ * Copyright (c) 1999,2000,2001,2002,2003,2023
  * Broadcom Corporation
  * All Rights Reserved.
  *---------------------------------------------------------------
@@ -54,7 +54,8 @@ struct histogram {
     unsigned int bincount;
     unsigned int binwidth;
     unsigned int populationcnt;
-    int final;
+    bool Omit;
+    bool final;
     int maxbin;
     int fmaxbin;
     double maxval;
@@ -73,7 +74,7 @@ struct histogram {
 };
 
 extern struct histogram *histogram_init(unsigned int bincount, unsigned int binwidth, float offset,\
-				   float units, double ci_lower, double ci_upper, unsigned int id, char *name);
+					float units, double ci_lower, double ci_upper, unsigned int id, char *name, bool omit);
 extern void histogram_delete(struct histogram *h);
 extern int histogram_insert(struct histogram *h, float value, struct timeval *ts);
 extern void histogram_clear(struct histogram *h);
