@@ -89,6 +89,7 @@ struct ReportStruct {
     int l2errors;
     int l2len;
     int expected_l2len;
+    u_char tos;
     // isochStartTime is overloaded: first write timestamp of the frame or burst w/trip-times or very first read w/o trip-times
     // reporter calculation will compute latency accordingly
     struct timeval isochStartTime;
@@ -99,6 +100,8 @@ struct ReportStruct {
     uint32_t remaining;
     bool transit_ready;
     int writecnt;
+    intmax_t writeLen;
+    intmax_t recvLen;
     long write_time;
     bool scheduled;
     long sched_err;
