@@ -93,12 +93,19 @@
 # define	IPTOS_DSCP_AF42		0x90
 # define	IPTOS_DSCP_AF43		0x98
 # define	IPTOS_DSCP_EF		0xb8
-# define	IPTOS_DSCP_LE		0x01 // rfc8622
-# define	IPTOS_DSCP_NQB		0x05 // rfc8325
-# define	IPTOS_DSCP_NQB2		0x2D // rfc8325 45 decimal
-
 #endif /* IPTOS_DSCP_AF11 */
 
+#ifndef IPTOS_DSCP_LE
+# define	IPTOS_DSCP_LE		0x01 // rfc8622
+#endif
+#ifndef IPTOS_DSCP_NQB
+# define	IPTOS_DSCP_NQB		0x05 // rfc8325
+#endif
+#ifndef IPTOS_DSCP_NQB2
+# define	IPTOS_DSCP_NQB2		0x2D // rfc8325 45 decimal
+#endif
+
+
 #ifndef IPTOS_DSCP_CS0
 # define	IPTOS_DSCP_CS0		0x00
 # define	IPTOS_DSCP_CS1		0x20
